//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/FTDecayFunctionItems.cpp
//! @brief     Implements FTDecayFunction1DItem classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Sample/FTDecayFunctionItems.h"
#include "Base/Const/Units.h"
#include "GUI/Model/XML/Serializer.h"
#include <QList>

// --------------------------------------------------------------------------------------------- //

FTDecayFunction1DItem::FTDecayFunction1DItem()
{
    m_decayLength.init("Decay length", "Decay length (half-width of the distribution)", 1000.0,
                       Unit::nanometer, "decay");
}

void FTDecayFunction1DItem::serialize(Serializer& s)
{
    s.assertVersion(0);
    s.rw(m_decayLength);
}

DoubleDescriptor FTDecayFunction1DItem::decayLength() const
{
    return m_decayLength;
}

DoubleDescriptors FTDecayFunction1DItem::valueDescriptors() const
{
    return {decayLength()};
}

// --------------------------------------------------------------------------------------------- //

std::unique_ptr<IFTDecayFunction1D> FTDecayFunction1DCauchyItem::createFTDecayFunction() const
{
    return std::make_unique<FTDecayFunction1DCauchy>(decayLength());
}

// --------------------------------------------------------------------------------------------- //

std::unique_ptr<IFTDecayFunction1D> FTDecayFunction1DGaussItem::createFTDecayFunction() const
{
    return std::make_unique<FTDecayFunction1DGauss>(decayLength());
}

// --------------------------------------------------------------------------------------------- //

std::unique_ptr<IFTDecayFunction1D> FTDecayFunction1DTriangleItem::createFTDecayFunction() const
{
    return std::make_unique<FTDecayFunction1DTriangle>(decayLength());
}

// --------------------------------------------------------------------------------------------- //

FTDecayFunction1DVoigtItem::FTDecayFunction1DVoigtItem()
{
    m_eta.init("Eta", "Parameter [0,1] to balance between Cauchy (eta=0.0) and Gauss (eta=1.0)",
               0.5, Unit::unitless, 3, RealLimits::limited(0.0, 1.0), "eta");
}

std::unique_ptr<IFTDecayFunction1D> FTDecayFunction1DVoigtItem::createFTDecayFunction() const
{
    return std::make_unique<FTDecayFunction1DVoigt>(decayLength(), m_eta);
}

void FTDecayFunction1DVoigtItem::serialize(Serializer& s)
{
    s.assertVersion(0);
    s.rw(m_decayLength);
    s.rw(m_eta);
}

DoubleDescriptor FTDecayFunction1DVoigtItem::eta() const
{
    return m_eta;
}

DoubleDescriptors FTDecayFunction1DVoigtItem::valueDescriptors() const
{
    return FTDecayFunction1DItem::valueDescriptors() + DoubleDescriptors{eta()};
}

// --------------------------------------------------------------------------------------------- //

FTDecayFunction2DItem::FTDecayFunction2DItem()
{
    m_decayLengthX.init(
        "Decay length X",
        "Decay length (half-width of the distribution) along x-axis of the distribution", 1000.0,
        Unit::nanometer, "decayX");
    m_decayLengthY.init(
        "Decay length Y",
        "Decay length (half-width of the distribution) along y-axis of the distribution", 1000.0,
        Unit::nanometer, "decayY");
    m_gamma.init("Gamma", "Distribution orientation with respect to the first lattice vector", 0.0,
                 Unit::degree, "gamma");
}

void FTDecayFunction2DItem::serialize(Serializer& s)
{
    s.assertVersion(0);
    s.rw(m_decayLengthX);
    s.rw(m_decayLengthY);
    s.rw(m_gamma);
}

DoubleDescriptor FTDecayFunction2DItem::decayLengthX() const
{
    return m_decayLengthX;
}

void FTDecayFunction2DItem::setDecayLengthX(const double len)
{
    m_decayLengthX.set(len);
}

DoubleDescriptor FTDecayFunction2DItem::decayLengthY() const
{
    return m_decayLengthY;
}

void FTDecayFunction2DItem::setDecayLengthY(const double len)
{
    m_decayLengthY.set(len);
}

DoubleDescriptor FTDecayFunction2DItem::gamma() const
{
    return m_gamma;
}

void FTDecayFunction2DItem::setGamma(const double gamma)
{
    m_gamma.set(gamma);
}

DoubleDescriptors FTDecayFunction2DItem::valueDescriptors() const
{
    return {decayLengthX(), decayLengthY(), gamma()};
}

// --------------------------------------------------------------------------------------------- //

std::unique_ptr<IFTDecayFunction2D> FTDecayFunction2DCauchyItem::createFTDecayFunction() const
{
    return std::make_unique<FTDecayFunction2DCauchy>(decayLengthX(), decayLengthY(),
                                                     Units::deg2rad(gamma()));
}

// --------------------------------------------------------------------------------------------- //

std::unique_ptr<IFTDecayFunction2D> FTDecayFunction2DGaussItem::createFTDecayFunction() const
{
    return std::make_unique<FTDecayFunction2DGauss>(decayLengthX(), decayLengthY(),
                                                    Units::deg2rad(gamma()));
}

// --------------------------------------------------------------------------------------------- //

FTDecayFunction2DVoigtItem::FTDecayFunction2DVoigtItem()
{
    m_eta.init("Eta", "Parameter [0,1] to balance between Cauchy (eta=0.0) and Gauss (eta=1.0)",
               0.5, Unit::unitless, 3, RealLimits::limited(0.0, 1.0), "eta");
}

std::unique_ptr<IFTDecayFunction2D> FTDecayFunction2DVoigtItem::createFTDecayFunction() const
{
    return std::make_unique<FTDecayFunction2DVoigt>(decayLengthX(), decayLengthY(), m_eta,
                                                    Units::deg2rad(gamma()));
}

void FTDecayFunction2DVoigtItem::serialize(Serializer& s)
{
    s.assertVersion(0);
    s.rw(m_decayLengthX);
    s.rw(m_decayLengthY);
    s.rw(m_gamma);
    s.rw(m_eta);
}

void FTDecayFunction2DVoigtItem::setEta(const double eta)
{
    m_eta.set(eta);
}

DoubleDescriptor FTDecayFunction2DVoigtItem::eta() const
{
    return m_eta;
}

DoubleDescriptors FTDecayFunction2DVoigtItem::valueDescriptors() const
{
    return {decayLengthX(), decayLengthY(), eta(), gamma()};
}
